using System.Net;
using Common;
using DotNext;
using WebProtocol;

namespace JtagController;

/// <summary>
/// Global Constant Jtag Address
/// </summary>
public static class JtagAddr
{
    /// <summary>
    /// Jtag State Reg
    /// </summary>
    public const UInt32 STATE = 0x10_00_00_00;
    /// <summary>
    /// Jtag Read Reg
    /// </summary>
    public const UInt32 READ_DATA = 0x10_00_00_01;
    /// <summary>
    /// Jtag Write Reg
    /// </summary>
    public const UInt32 WRITE_DATA = 0x10_00_00_02;
    /// <summary>
    /// Jtag Write Command
    /// </summary>
    public const UInt32 WRITE_CMD = 0x10_00_00_03;
}

class Jtag
{
    readonly int timeout;

    readonly int port;
    readonly string address;
    private IPEndPoint ep;
    private UDPServer server;

    public Jtag(string address, int port, UDPServer udpServer, int outTime = 2000)
    {
        this.address = address;
        this.port = port;
        this.ep = new IPEndPoint(IPAddress.Parse(address), port);
        this.server = udpServer;
        this.timeout = outTime;
    }

    public async ValueTask<Result<bool>> ClearRegisters()
    {
        var ret = true;
        var opts = new SendAddrPackOptions();

        opts.burstType = BurstType.FixedBurst;
        opts.burstLength = 4;
        opts.commandID = 0;
        opts.address = JtagAddr.STATE;

        // Write Jtag State Register
        opts.isWrite = true;
        ret = await UDPClientPool.SendAddrPackAsync(ep, new SendAddrPackage(opts));
        if (!ret) throw new Exception("Send 1st Address Package Failed!");
        ret = await UDPClientPool.SendDataPackAsync(ep,
                new SendDataPackage(NumberProcessor.NumberToBytes(0xFF_FF_FF_FF, 4).Value));
        if (!ret) throw new Exception("Send Data Package Failed!");

        // Read Jtag State Register
        ret = await UDPClientPool.SendAddrPackAsync(ep, new SendAddrPackage(opts));
        if (!ret) throw new Exception("Send 2rd Address Package Failed!");

        // Wait for Ack
        var data = await UDPServer.


        return ret;
    }
}
